# MD001.r: Simulate Three Class Model with Random Intercept Within Class
# Class Covariate = Gender
# Five Time points, Linear Time Effect
# Random Intercepts only
# Outfiles:  G:/Staff_Users/Brian/LC/sim09/data/simdata.txt
# Dec 2009
######################
set.seed(10010)
library(mvtnorm)
library(nnet)

n<-600
ni<-10
t<-matrix(rep(0:(ni-1),n),n,ni,byrow=T)	# Time

# Class covariates  and parms
male<-rbinom(n,1,.65)	# Male
W<-cbind(rep(1,n),male)
g2<-c(-.5,.5)		# Class 2 Parms
g3<-c(.75,-.75)		# Class 3 Parms

eta2<-W%*%g2
eta3<-W%*%g3

expeta2<-exp(eta2)
expeta3<-exp(eta3)

p1<-1/(1+expeta2+expeta3)
p2<-p1*expeta2
p3<-p1*expeta3
p<-cbind(p1,p2,p3)

# CLASS INDICATORS 
c<-rep(0,n)
for (i in 1:n) c[i]<-(1:3)[rmultinom(1,1,p[i,])==1]
#table(c)
#summary(multinom(c~male))

n1<-length(c[c==1])
n2<-length(c[c==2])
n3<-length(c[c==3])

y1<-matrix(NA,n,ni)  # Occurrence
y2<-matrix(NA,n,ni)  # Intensity

 ############
 #  Class 1 #
 ############
 # Random Effect
   cov1<-matrix(c(2,.5,.5,2),ncol=2)  # Random Effects Cov
   u1<-rmvnorm(n1,sigma=cov1)
   u11<-u1[,1]
   u12<-u1[,2]

  # Occurrence
    a1<-c(.25,.5)
    eta<-a1[1]+a1[2]*t[c==1,]+u11 
    p<-pnorm(eta)
    y1[c==1,]<-rbinom(n1*ni,1,p) # Occurrence Response

  # Intensity
    sigma1<-.5
    b1<-c(3.25,-.75)
    eta<-b1[1]+b1[2]*t[c==1,]+u12 
    y2[c==1,]<-rlnorm(n1*ni,eta,sigma1)
  
 ############
 #  Class 2 #
 ############ 
 # Random Effect
   cov2<-matrix(c(1,-.25,-.25,1),ncol=2)  # Random Effects Cov
   u2<-rmvnorm(n2,sigma=cov2)
   u21<-u2[,1]
   u22<-u2[,2]

   a2<-c(.5,-.5)
   eta<-a2[1]+a2[2]*t[c==2,]+u21
   p<-pnorm(eta)
   y1[c==2,]<-rbinom(n2*ni,1,p) # Occurrence Response

  # Intensity
  sigma2<-1
  b2<-c(2.25,1.25)
  eta<-b2[1]+b2[2]*t[c==2,]+u22
  y2[c==2,]<-rlnorm(n2*ni,eta,sigma2)

 ############
 #  Class 3 #
 ############
 # Random Effect
   cov3<-matrix(c(1.5,0,0,.5),ncol=2)  # Random Effects Cov
   u3<-rmvnorm(n3,sigma=cov3)
   u31<-u3[,1]
   u32<-u3[,2]

  # Occurrence
   a3<-c(-.5,.5)
   eta<-a3[1]+a3[2]*t[c==3,]+u31
   p<-pnorm(eta)
   y1[c==3,]<-rbinom(n3*ni,1,p) # Occurrence Response
   y1vec<-c(t(y1))

  # Intensity
  sigma3<-1.5
  b3<-c(.5,.5)
  eta<-b3[1]+b3[2]*t[c==3,]+u32
  y2[c==3,]<-rlnorm(n3*ni,eta,sigma3)

#########
#  SAVE #
#########
  y2[y1==0]<-0					# Set y2=0 if y1=0
  y2vec<-c(t(y2))
  tvec<-c(t(t))
  b<-matrix(0,n,2)
  b[c==1,]<-u1
  b[c==2,]<-u2
  b[c==3,]<-u3
  data<-cbind(y=y2vec,tvec,c=rep(c,each=ni),male=rep(male,each=ni))

write.table(data,file="C:/Users/chech/Desktop/Research/Bayesian Two-Part/simdata3.txt",
	quote=FALSE, row.names=FALSE, col.names=FALSE)







