# Three Class MCMC.r: Fit Three Class Model
# NOTE: True Model has 3 Classes
# INFILE: G:/Staff_Users/Brian/LC/sim09/data/simdata.txt
# Outfile: G:/Staff_Users/Brian/LC/sim09/output/results.txt
# Dec 2009
#############################
library(mvtnorm)
library(Hmisc)		# for rMultinom fn used in update.c
library(MCMCpack) 	# iwish distribution used in updating Sigma

#############################
#  Source in MCMC Functions #
#############################
source ("C:/Users/chech/Desktop/Research/Bayesian Two-Part/Update Gamma.r")
source ("C:/Users/chech/Desktop/Research/Bayesian Two-Part/Update C Random Intercept.r")
source ("C:/Users/chech/Desktop/Research/Bayesian Two-Part/Update Alpha Random Intercept.r")
source ("C:/Users/chech/Desktop/Research/Bayesian Two-Part/Update Beta Random Intercept.r")
source ("C:/Users/chech/Desktop/Research/Bayesian Two-Part/Update b.r")

###########
#   Data  #
###########
data<-read.table(file="C:/Users/chech/Desktop/Research/Bayesian Two-Part/simdata3.txt")
N<-nrow(data)			# Number of observations
ni<-10					# Number of obs/subject
K<-3					# Number of Classes (NOTE: Must be >= 2)
n<-N/ni				# Number of subjects
y<-ly<-data[,1]			# y and log(y)
Y<-matrix(y,n,ni,byrow=T)	
ly[y>0]<-log(y[y>0])		# ly=Log(y)--note: will only use when y>0

t<-data[,2]				# Linear Time
X<-cbind(rep(1,N),t)		# Design Matrix
p<-ncol(X)				# Number of Parms

true.c<-data[t==0,3]		# True Class Indicators
w<-data[t==0,4]			# Class Covariate	
W<-cbind(rep(1,n),w)		# Class Design Matrix
r<-ncol(W)	

nsim<-10000				# Number of MCMC Iterations (10K ~15 min) 
thin<-5
burn<-nsim/2
lastit<-(nsim-burn)/thin	# Last stored value

##########
# Priors #
##########
 m0<-rep(0,p)			# Prior Mean for alpha and beta, all classes
 v0<-diag(10,p)			# Prior Cov of Alpha & beta, all classes
 prec0<-solve(v0)
 lambda<-delta<-.01		# Hyperpriors for tau1 - tau3

###################
# Initial Values	#
###################
 c<-rep(0,n)			# Latent class indicators

# Set Inits at 0 in Practice
  gamma<-matrix(0,(K-1),r)	# Class indicator parms
  alpha<-beta<-matrix(0,K,p)	# Component 1 regression parms 

# Here, I set inits at true values, which "encourages" labels to line up with true vals,
# making it easier to read the output below...but, obviously, this is not possible in practice
   gamma<-rbind(c(-.5,.5),c(.75,-.75))	
   alpha<-rbind(c(.25,.5),c(.5,-.5),c(-.5,.5))
   beta<-rbind(c(3.25,-.75),c(2.25,1.25),c(.5,.5))

 tausq<-rep(1,K)			# Lognormal variances
 tauinv<-1/tausq			# Lognormal Precisions
 b<-matrix(0,n,2)
 Sigma<-diag(K*2)

#########################
#   STORE RESULTS		#
#########################
 
 Alpha<-Beta<-matrix(0,lastit,p*K) 		# Store Parm Results, both classes
 Tau<-matrix(0,lastit,K)			# Lognormal SDs
 Gamma<-matrix(0,lastit,(K-1)*r)	
 Gtmp<-matrix(0,nsim,(K-1)*r)			# Store gamma at each it. for update gamma
 Sigma1<-Sigma2<-Sigma3<-matrix(0,lastit,4)		
 Cs<-B1<-B2<-matrix(0,lastit,n)		# Store class indicators and Rand Fx

###################
#      MCMC		#
###################
set.seed(010101)
ptm<-proc.time()

for (i in 1:nsim) {
  # UPDATE Gamma
    gamma<-update.gamma(g=gamma,W=W,c=c,s=100,burn=nsim/4,it=i,G=Gtmp)
    
  # UPDATE class indicators, c
    c<-update.c(g=gamma,y,W,X,b,alpha,beta,tausq,Sigma)
    C<-rep(c,each=ni)			# Concatenate to length N

  # UPDATE alpha, beta for each class
   for (k in 1:K) {
     alpha[k,]<-update.alpha(y=y[C==k],X=X[C==k,],alpha=alpha[k,],
				     b1=b[c==k,1],m0=m0,prec0=prec0) 
     beta[k,]<-update.beta(y=y[C==k],X=X[C==k,],b2=b[c==k,2],
			      m0=m0,prec0=prec0,tinv=tauinv[k])

  # UPDATE tauinv_k, lognormal precision, for class k
      mtau<-lambda+length(y[y>0&C==k])/2         
      zb<-rep(b[,2],each=ni)	   		# Note: Only zb[C_i=k] used below   
      tauinv[k]<-rgamma(1,mtau,delta+sum((ly-X%*%beta[k,]-zb)[C==k&y>0,]^2)/2)
   
  # UPDATE  b_k
      b[c==k,]<-update.b(b[c==k,],
			Sigma=Sigma[((k-1)*2+1):(2*k),((k-1)*2+1):(2*k)],
			y=y[C==k],X=X[C==k,],alpha=alpha[k,],
			beta=beta[k,],tausq=1/tauinv[k])

  # UPDATE Sigma_k
   	Sigma[((k-1)*2+1):(2*k),((k-1)*2+1):(2*k)]<- 
		riwish(nrow(b[c==k,])+2, diag(2)+crossprod(b[c==k,],b[c==k,]))
   }
   
   tausq<-1/tauinv				# Lognormal variances

 # Store Results
 if (i> burn & i%%thin==0) {
  j<-(i-burn)/thin

  Alpha[j,]<-c(t(alpha))
  Beta[j,]<-c(t(beta))
  Tau[j,]<-sqrt(tausq)				# Lognormal SDs
  Gamma[j,]<-c(t(gamma))
  Sigma1[j,]<-c(Sigma[1:2,1:2])
  Sigma2[j,]<-c(Sigma[3:4,3:4])
  Sigma3[j,]<-c(Sigma[5:6,5:6])
  Cs[j,]<-c
  B1[j,]<-b[,1]
  B2[j,]<-b[,2] 
 }
 Gtmp[i,]<-c(t(gamma))			# Store every it. for updating gamma (may want to revise)
 if (i%%100==0) print(i)

} 		# End MCMC

proc.time()-ptm

# RESULTS
 mean.alpha<-apply(Alpha,2,mean)
 mean.beta<-apply(Beta,2,mean)
 mean.tau<-apply(Tau,2,mean)
 mean.gamma<-apply(Gamma,2,mean)
 mean.c<-apply(Cs,2,mean)
 mean.sigma1<-apply(Sigma1,2,mean)
 mean.sigma2<-apply(Sigma2,2,mean)
 mean.sigma3<-apply(Sigma3,2,mean)

cat("True Alpha = ",c(.25,.5,.5,-.5,-.5,.5),"\n","Post Mean = ", mean.alpha,"\n")
cat("True Beta = ",c(3.25,-.75,2.25,1.25,.5,.5),"\n","Post Mean = ", mean.beta,"\n")
cat("True Gamma = ",c(-.5,.5,.75,-.75),"\n","Post Mean = ", mean.gamma,"\n")
cat("True Sigma1 = ",c(2,.5,.5,2),"\n","Post Mean = ", mean.sigma1,"\n")
cat("True Sigma2 = ",c(1,-.25,-.25,1),"\n","Post Mean = ", mean.sigma2,"\n")
cat("True Sigma3 = ",c(1.5,0,0,.5),"\n","Post Mean = ", mean.sigma3,"\n")

plot(1:lastit,Alpha[,5],type="l",col="lightgreen")
abline(h=mean(Alpha[,5]),col="blue")


# 95% CI
ci.alpha<-apply(Alpha,2,function(x) quantile(x,probs = c(0.025,0.975)))
ci.beta<-apply(Beta,2,function(x) quantile(x,probs = c(0.025,0.975)))
ci.tau<-apply(Tau,2,function(x) quantile(x,probs = c(0.025,0.975)))
ci.gamma<-apply(Gamma,2,function(x) quantile(x,probs = c(0.025,0.975)))
ci.c<-apply(Cs,2,function(x) quantile(x,probs = c(0.025,0.975)))
ci.sigma1<-apply(Sigma1,2,function(x) quantile(x,probs = c(0.025,0.975)))
ci.sigma2<-apply(Sigma2,2,function(x) quantile(x,probs = c(0.025,0.975)))
ci.sigma3<-apply(Sigma3,2,function(x) quantile(x,probs = c(0.025,0.975)))

cat("True Alpha = ",c(.25,.5,.5,-.5,-.5,.5),"\n","95% CI lower bound= ", ci.alpha[1,1:6],"\n","95% CI upper bound= ",ci.alpha[2,1:6],"\n")
cat("True Beta = ",c(3.25,-.75,2.25,1.25,.5,.5),"\n","95% CI lower bound= ", ci.beta[1,1:6],"\n","95% CI upper bound= ",ci.beta[2,1:6],"\n")
cat("True Gamma = ",c(-.5,.5,.75,-.75),"\n","95% CI lower bound= ", ci.gamma[1,1:4],"\n","95% CI upper bound= ",ci.gamma[2,1:4],"\n")
cat("True Sigma1 = ",c(2,.5,.5,2),"\n","95% CI= lower bound", ci.sigma1[1,1:4],"\n","95% CI upper bound= ",ci.sigma1[2,1:4],"\n")
cat("True Sigma2 = ",c(1,-.25,-.25,1),"\n","95% CI= lower bound", ci.sigma2[1,1:4],"\n","95% CI upper bound= ",ci.sigma2[2,1:4],"\n")
cat("True Sigma3 = ",c(1.5,0,0,.5),"\n","95% CI= lower bound", ci.sigma3[1,1:4],"\n","95% CI upper bound= ",ci.sigma3[2,1:4],"\n")



############# OUTPUT Results #############
results<-list(alpha=Alpha,beta=Beta,tau=Tau,b1=B1,b2=B2,Sigma1=Sigma1,
		  Sigma2=Sigma2,Sigma3=Sigma3,gamma=Gamma,c=Cs)
#dput(results,file="G:/Staff_Users/Brian/LC/sim09/output/results.txt")


