# Update alpha.r using Albert & Chib probit model
# ADDS Random Intercept
# Function Inputs:
#	y  = N_k x 1 response vector
#	X  = N_k x p design matrix
#	alpha = current vector of alpha parms for class k (k known)
#     b1 = n_k x 1 vector of random intercepts for Bernoulli compnt., class k
#	m0 = prior mean
#	prec0= prior precision	
#
# April 24, 2009
##################################################  
update.alpha<-function(y=y[C==k],X=X[C==k,],alpha=alpha[k,],
				b1,m0=m0,prec0=prec0) {
   n<-length(b1)
   N<-length(y)
   ni<-N/n 
   u<-rep(0,N)
   muu<-X%*%alpha+rep(b1,each=ni)		# Update Mean of u; alpha[k,] is current value
   u[y==0]<-qnorm(runif(N,0,pnorm(0,muu,1)),muu,1)[y==0]
   u[y>0]<-qnorm(runif(N,pnorm(0,muu,1),1),muu,1)[y>0]   
   valpha<-solve(prec0+crossprod(X,X))	# This could be moved outside of MCMC
   malpha <- valpha%*%(prec0%*%m0+crossprod(X,u-rep(b1,each=ni)))

  # Draw Alpha from its full conditional(s) and Store
    return(c(rmvnorm(1,malpha,valpha)))
}