# Update C3 (Class Indicators and Corresponding Sample Sizes, N1, N2, N3)
# NOTE: MUST LOAD Hmisc Library for rMultinom function
# NOTE 2: Assumes 3 random intercepts per component per subject (1 for each class)

# Function Parameters:
  # gamma = (K-1) x r matrix of class-membership reg. parameters (K and r >= 2)
  #  		 note 1: last gamma=rep(0,r) for identification purposes
  #		 note 2: if r=1 can use Dirichlet prior of class indicators
  # y     = N x 1 response vector 
  # W     = n x r matrix of covariates including intercept
  # X     = N x p matrix of time covariates (including intercept)
  # alpha = K x p matrix of component-1 coefficients [for (Pr(Y>0)]
  # beta  = K x p matrix of component-2 coefs [for lognormal component]
  # b     = n x 2 matrix of random effects
  # tausq = K x 1 vector of lognormal variances (NOT PRECISIONS)
  # Sigma = 2K x 2K random effect covariance

# May 21, 2009
#######################

update.c<-function(g=gamma,y=y,W=W,X=X,b=b,
			 alpha=alpha,beta=beta,tausq=tausq,Sigma=Sigma) {
 
 n<-nrow(W)				# Number of subjects
 r<-ncol(W)				# Number of class reg parms, incl intcpt
 N<-nrow(X)				# Total Number of Observations
 ni<-N/n				# Number of obs. per subject (assumed the same)
 Y<-matrix(y,n,ni,byrow=T)
 b1<-rep(b[,1],each=ni)
 b2<-rep(b[,2],each=ni)
 if (is.vector(g)==TRUE) g<-matrix(g,1,ncol=r)	# Convert g to matrix if vector
 K<-nrow(g)+1  			# Number of classes
 
 eta<-cbind(rep(0,n),W%*%t(g))
 pi<-exp(eta)/(1+apply(as.matrix(exp(eta[,-1])),1,sum)) # Class proportions, given W

 phi<-pnorm(X%*%t(alpha)+b1)			# Prob y>0 (Cols are classes 1 to K)
 mu<-X%*%t(beta)+b2				# Lognormal mean
 tau<-sqrt(tausq)					# Lognormal SD for classes 1-K

 # Likelihood contributions, Classes 1-K
   l<-matrix(0,N,K)
   L<-array(0,dim=c(n,ni,K))			# Array to store lklhoods for 1-K;
							#   used to get column-wise product
   pnum<-matrix(0,n,K)				# Numerator for post prob of C_i=k

 for (k in 1:K) {
  l[y==0,k]<-1-phi[y==0,k]			# Binom like 	
  l[y>0,k]<-phi[y>0,k]*dlnorm(y[y>0],mu[y>0,k],tau[k]) # LN like 
  L[,,k]<-matrix(l[,k],n,ni,byrow=T)
  sigma<-Sigma[((k-1)*2+1):(2*k),((k-1)*2+1):(2*k)]
  pnum[,k]<-pi[,k]*apply(L[,,k],1,prod)*dmvnorm(b,sigma=sigma) # need prior given c_i=k
 }
 p<-pnum/(apply(pnum,1,sum))
 c<-rMultinom(p,1)				# 1 draw for each row w.p row(p) 

 return(c)
}




