# Update b.r
# Update n x 2 random effects using Metropolis
# ASSUMES RANDOM INTERCEPTS ONLY
# Inputs:
#	b  = n_k x 2 matrix of random effects for subject in class k
#	Sigma= 2 x 2 Estimate of Posterior Cov (Default is diag (2) or prior cov)
#	y = N_k x 1 response vector
#	X = N_k x p fixed effects design matrix (including intercept)
#     alpha = p x 1 vector of Bernoulli fixed effects for class k 
#     beta = p x 1 vector of lognormal fixed effects for class k
#     tausq  = lognormal Variance for class k
#     cov = proposal covariance (USING SIGMA FOR NOW, SO NOT INCLUDED)
# RETURNS:  n_k x 2 matrix of random effects for class k
#		NOTE: n_k = current sample size for class k
# April 28, 2009
###################################

update.b<-function(b=b[c==k,],Sigma=diag(2),y=y[C==k],
			 X=X[C==k,],alpha=alpha[k,],
			 beta=beta[k,],tausq=1/tauinv[k]) {
   N<-length(y)
   n<-nrow(b)
   ni<-N/n
   l<-ls<-rep(0,N)

  # Current log like_i
   eta<-X%*%alpha+rep(b[,1],each=ni)
   p<-pnorm(eta)
   mu<-(X%*%beta+rep(b[,2],each=ni))[y>0]
   l[y==0]<-log(1-p[y==0])			# current log like
   l[y>0]<-log(p[y>0]*dlnorm(y[y>0],mu,sqrt(tausq)))
  
 # Draw Candidate from Symmetric Rand Walk Proposal 
   bs<-b+rmvnorm(n,sigma=Sigma)  		# or b+rmvt(sigma=Sigma,3)
   etas<-X%*%alpha+rep(bs[,1],each=ni)
   ps<-pnorm(etas)
   mus<-(X%*%beta+rep(bs[,2],each=ni))[y>0]  
   ls[y==0]<-log(1-ps[y==0])			# candidate log like
   ls[y>0]<-log(ps[y>0]*dlnorm(y[y>0],mus,sqrt(tausq)))
 
   Ls<-apply(matrix(ls,n,ni,byrow=ni),1,sum)
   L<-apply(matrix(l,n,ni,byrow=ni),1,sum)
   
   ratio<-Ls+dmvnorm(bs,c(0,0),Sigma,log=TRUE)-(L+dmvnorm(b,c(0,0),Sigma,log=TRUE))
		
   u<-1*(log(runif(n))<ratio)
   b[u==1,]<-bs[u==1,]
   return(b)
}